@@ -30,13 +30,12 @@
 
 #include "kerfuffle_export.h"
 
-#include <QString>
 #include <QHash>
+#include <QMimeType>
 #include <QObject>
 #include <QVariant>
-#include <QDebug>
 
-#include <KServiceTypeTrader>
+#include <KPluginMetaData>
 
 class KJob;
 
@@ -135,25 +134,72 @@ QDebug operator<<(QDebug d, const fileRootNodePair &pair);
 class KERFUFFLE_EXPORT Archive : public QObject
 {
     Q_OBJECT
+    Q_ENUMS(EncryptionType)
+
+    /**
+     *  Complete base name, without the "tar" extension (if any).
+     */
+    Q_PROPERTY(QString completeBaseName READ completeBaseName CONSTANT)
+    Q_PROPERTY(QString fileName READ fileName CONSTANT)
+    Q_PROPERTY(QString comment READ comment CONSTANT)
+    Q_PROPERTY(QMimeType mimeType READ mimeType CONSTANT)
+    Q_PROPERTY(bool isReadOnly READ isReadOnly CONSTANT)
+    Q_PROPERTY(bool isSingleFolderArchive READ isSingleFolderArchive)
+    Q_PROPERTY(EncryptionType encryptionType READ encryptionType)
+    Q_PROPERTY(qulonglong numberOfFiles READ numberOfFiles)
+    Q_PROPERTY(qulonglong unpackedSize READ unpackedSize)
+    Q_PROPERTY(qulonglong packedSize READ packedSize)
+    Q_PROPERTY(QString subfolderName READ subfolderName)
 
 public:
 
-    static bool comparePlugins(const KService::Ptr &p1, const KService::Ptr &p2);
-    static QString determineMimeType(const QString& filename);
-    static KService::List findPluginOffers(const QString& filename, const QString& fixedMimeType);
+    enum EncryptionType
+    {
+        Unencrypted,
+        Encrypted,
+        HeaderEncrypted
+    };
+
+    QString completeBaseName() const;
+    QString fileName() const;
+    QString comment() const;
+    QMimeType mimeType() const;
+    bool isReadOnly() const;
+    bool isSingleFolderArchive();
+    bool hasComment() const;
+    EncryptionType encryptionType();
+    qulonglong numberOfFiles();
+    qulonglong unpackedSize();
+    qulonglong packedSize() const;
+    QString subfolderName();
+
+    static bool comparePlugins(const KPluginMetaData &p1, const KPluginMetaData &p2);
+    static QVector<KPluginMetaData> findPluginOffers(const QString& filename, const QString& fixedMimeType);
 
     static Archive *create(const QString &fileName, QObject *parent = 0);
     static Archive *create(const QString &fileName, const QString &fixedMimeType, QObject *parent = 0);
+
+    /**
+     * Create an archive instance from a given plugin.
+     * @param fileName The name of the archive.
+     * @param pluginMetadata The plugin's metadata.
+     * @return A valid archive if the plugin could be loaded, an invalid one otherwise (with the FailedPlugin error set).
+     */
+    static Archive *create(const QString &fileName, const KPluginMetaData &pluginMetadata, QObject *parent = Q_NULLPTR);
+
     ~Archive();
 
     ArchiveError error() const;
     bool isValid() const;
-    QString fileName() const;
-    bool isReadOnly() const;
 
     KJob* open();
     KJob* create();
+
+    /**
+     * @return A ListJob if the archive already exists. A null pointer otherwise.
+     */
     ListJob* list();
+
     DeleteJob* deleteFiles(const QList<QVariant> & files);
 
     /**
@@ -172,18 +218,17 @@ public:
 
     ExtractJob* copyFiles(const QList<QVariant> &files, const QString &destinationDir, const ExtractionOptions &options = ExtractionOptions());
 
-    bool isSingleFolderArchive();
-    QString subfolderName();
-    bool isPasswordProtected();
-
-    void setPassword(const QString &password);
-    void enableHeaderEncryption(bool enable);
-    QString comment() const;
+    /**
+     * @param password The password to encrypt the archive with.
+     * @param encryptHeader Whether to encrypt also the list of files.
+     */
+    void encrypt(const QString &password, bool encryptHeader);
 
 private slots:
     void onListFinished(KJob*);
     void onAddFinished(KJob*);
     void onUserQuery(Kerfuffle::Query*);
+    void onNewEntry(const ArchiveEntry &entry);
 
 private:
     Archive(ReadOnlyArchiveInterface *archiveInterface, bool isReadOnly, QObject *parent = 0);
@@ -193,20 +238,18 @@ private:
     ReadOnlyArchiveInterface *m_iface;
     bool m_hasBeenListed;
     bool m_isReadOnly;
-    bool m_isPasswordProtected;
     bool m_isSingleFolderArchive;
 
     QString m_subfolderName;
-    qlonglong m_extractedFilesSize;
+    qulonglong m_extractedFilesSize;
     ArchiveError m_error;
+    EncryptionType m_encryptionType;
+    qulonglong m_numberOfFiles;
 };
 
-KERFUFFLE_EXPORT QSet<QString> supportedMimeTypes();
-KERFUFFLE_EXPORT QSet<QString> supportedWriteMimeTypes();
-KERFUFFLE_EXPORT QSet<QString> supportedEncryptEntriesMimeTypes();
-KERFUFFLE_EXPORT QSet<QString> supportedEncryptHeaderMimeTypes();
 } // namespace Kerfuffle
 
+Q_DECLARE_METATYPE(Kerfuffle::Archive::EncryptionType)
 Q_DECLARE_METATYPE(Kerfuffle::fileRootNodePair)
 
 #endif // ARCHIVE_H
