@@ -63,7 +63,7 @@ public:
     QMimeData * mimeData(const QModelIndexList & indexes) const Q_DECL_OVERRIDE;
     bool dropMimeData(const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent) Q_DECL_OVERRIDE;
 
-    KJob* setArchive(Kerfuffle::Archive *archive, bool existingArchive = false);
+    KJob* setArchive(Kerfuffle::Archive *archive);
     Kerfuffle::Archive *archive() const;
 
     Kerfuffle::ArchiveEntry entryForIndex(const QModelIndex &index);
@@ -75,8 +75,11 @@ public:
     Kerfuffle::AddJob* addFiles(const QStringList & paths, const Kerfuffle::CompressionOptions& options = Kerfuffle::CompressionOptions());
     Kerfuffle::DeleteJob* deleteFiles(const QList<QVariant> & files);
 
-    void setPassword(const QString &password);
-    void enableHeaderEncryption();
+    /**
+     * @param password The password to encrypt the archive with.
+     * @param encryptHeader Whether to encrypt also the list of files.
+     */
+    void encryptArchive(const QString &password, bool encryptHeader);
 
 signals:
     void loadingStarted();
